/**
 * Copyright Intermesh
 *
 * This file is part of Group-Office. You should have received a copy of the
 * Group-Office license along with Group-Office. See the file /LICENSE.TXT
 *
 * If you have questions write an e-mail to info@intermesh.nl
 *
 * @version $Id: StandardTaskGrid.js 23462 2018-03-06 11:37:47Z mschering $
 * @copyright Copyright Intermesh
 * @author Michael de Hart <mdhart@intermesh.nl>
 */
Ext.define('go.modules.business.business.ActivityGrid', {
	extend: go.grid.GridPanel,

	initComponent : function(){
		Ext.apply(this,{
			//title:t("Activities", "projects2"),
			tbar: [{
				xtype: 'tbtitle',
				text: t("Activities")
			},'->',{
				iconCls: "ic-add",
				handler: function() {
					var dlg = new go.modules.business.business.ActivityDialog();
					dlg.show();
				},scope: this
			}],
			store: new go.data.Store({
				fields: [
					'id',
					{name: 'code', type: 'string'},
					{name: 'type', type: 'int'},
					{name: 'name', type: 'string'},
					{name: 'description', type: 'string'},
					{name: 'units', type: 'float'},
					{name: 'billable', type: 'int'},
					{name: 'budgetable', type: 'boolean'},
					{name: 'disabled', type: 'boolean'}
				],
				sortInfo :{field: "name"},
				entityStore: "Activity"
			}),
			listeners: {
				rowdblclick: function (grid, rowIndex, e) {
					var record = grid.getStore().getAt(rowIndex);
					var dlg = new go.modules.business.business.ActivityDialog();
					dlg.load(record.id).show();
				},
				scope:this
			},
			viewConfig: {
				getRowClass : function(row, index) {
					if (row.data.disabled === true) {
						return 'go-grid-row-inactive';
					}
				}
			},
			border: false,
			paging:true,
			autoExpandColumn: 'description',
			columns:[{
				header: t("Code", "projects2"),
				dataIndex: 'code',
				hidden:true,
				width: 60
			},{
				header: t("Name"),
				dataIndex: 'name',
				width: 200,
				renderer: function(v,meta,r) {
					return '<i class="icon">'+ (r.data.type ==1 ? 'business_center' : r.data.type==2 ? 'beach_access' : 'transfer_within_a_station')+'</i> '+v;
				}
			},{
				header: t("Description"),
				id: 'description',
				dataIndex: 'description',
				renderer: function(value){
					return value.replace(/<br[^>]*>/gi," "); //hide those newlines in the grid
				}
			},{
				header: t("Duration"),
				dataIndex: 'units',
				width: 70,
				align:'right',
				renderer: function(v) { return v+' '+t('Hours');}
			},{
				header: t("Billable"),
				dataIndex: 'billable',
				width: 80,
				renderer: function(v) {return v==2 ? t('Always') : (v ? t('Yes') : t('No')); }
			},{
				header: t("Budgetable"),
				dataIndex: 'billable',
				width: 60,
				renderer: go.grid.ColumnRenderers.check
			}]
		});
		this.callParent();
		this.store.load();
	}
});
